@@ -49,6 +49,10 @@
 #pod     __PACKAGE__->meta->make_immutable;
 #pod     1;
 #pod
+#pod =cut
+
+# --------------------------------------------------------------------------------------------------
+
 #pod =head1 DESCRIPTION
 #pod
 #pod C<Dist::Zilla> limits logging capabilities with 3 logging levels available in plugings through
@@ -83,8 +87,8 @@
 #pod         $self->log_fatal( 'Aborting...' );
 #pod     };
 #pod
-#pod This works, but incrementing the counter is boring and error-prone. <Dist-Zilla-Role-ErrorLogger>
-#pod role automates it, making plugin code shorter and more readable:
+#pod This works, but incrementing the counter after each C<log> call is boring and error-prone.
+#pod C<Dist-Zilla-Role-ErrorLogger> role automates it, making plugin code shorter and more readable:
 #pod
 #pod     with 'Dist::Zilla::Role::ErrorLogger';
 #pod     …
@@ -94,12 +98,16 @@
 #pod
 #pod =cut
 
+# --------------------------------------------------------------------------------------------------
+
 package Dist::Zilla::Role::ErrorLogger;
 
+use feature qw{ state };
+
 # ABSTRACT: Provide error logging capabilities for Dist::Zilla plugins
 
 BEGIN {
-    our $VERSION = '0.001'; # VERSION
+    our $VERSION = '0.002'; # VERSION
 };
 
 use Moose::Role;
@@ -135,11 +143,39 @@ has error_count => (
 #pod         …
 #pod     };
 #pod
+#pod Messages are reported at C<error> level, if level is not explicitly specified. (It seems level is
+#pod not used by C<Dist::Zilla> now.) Custom arguments are respected, too:
+#pod
+#pod     $self->log_error( { prefix => 'subsystem: ', level => 'info' }, $message );
+#pod
+#pod (See L<Log::Dispatchouli/log> for more details. Err… Unfortunately, there are very little details
+#pod there.)
+#pod
 #pod =cut
 
 sub log_error {
     my $self = shift( @_ );
-    $self->log( @_ );
+    #   If the first argument is a hashref, it is treated as extra arguments to logging, not as
+    #   message, see <https://metacpan.org/pod/Log::Dispatchouli#log>. Let us add `level` argument.
+    #   However, we should not change user-supplied hash and arguments.
+    state $level = {
+        level => 'error',
+    };
+    my $arg;
+    if ( ref( $_[ 0 ] ) eq 'HASH' ) {
+        #   User provided hashref, remove it from `@_`.
+        $arg = shift( @_ );
+        #   Do not override user-supplied `level` argument, if any.
+        if ( not exists( $arg->{ level } ) ) {
+            #   User did not provide `level` argument, so we need to add it.
+            #   Note: new hash created intentionally, to keep user-supplied hash intact.
+            $arg = { %$level, %$arg };
+        };
+    } else {
+        #   User did not provide hashref, just use ours.
+        $arg = $level;
+    };
+    $self->log( $arg, @_ );
     ++ $self->{ error_count };
     return $self->{ error_count };
 };
@@ -149,14 +185,16 @@ sub log_error {
 #pod =method abort_if_error
 #pod
 #pod If there was any errors (i. e. C<error_count> is greater than zero), the method aborts execution by
-#pod calling C<log_fatal>.
+#pod calling C<log_fatal> with all the arguments. If the method is called with no arguments, it pass
+#pod to C<logh_fatal> string "Aborting...".
 #pod
 #pod =cut
 
 sub abort_if_error {
     my $self = shift( @_ );
     if ( $self->error_count ) {
-        $self->log_fatal( [ '%d errors, aborting...', $self->error_count ] );
+        $self->log_fatal( @_ ? @_ : 'Aborting...' );
+            #   `@_ or 'Aborting...'` looks shorter, but it evaluates `@_` in scalar context.
     };
 };
 
@@ -176,7 +214,7 @@ Dist::Zilla::Role::ErrorLogger - Provide error logging capabilities for Dist::Zi
 
 =head1 VERSION
 
-Version 0.001.
+Version 0.002.
 
 =head1 SYNOPSIS
 
@@ -238,8 +276,8 @@ errors, and calling C<log_fatal> once at the end:
         $self->log_fatal( 'Aborting...' );
     };
 
-This works, but incrementing the counter is boring and error-prone. <Dist-Zilla-Role-ErrorLogger>
-role automates it, making plugin code shorter and more readable:
+This works, but incrementing the counter after each C<log> call is boring and error-prone.
+C<Dist-Zilla-Role-ErrorLogger> role automates it, making plugin code shorter and more readable:
 
     with 'Dist::Zilla::Role::ErrorLogger';
     …
@@ -266,10 +304,19 @@ attribute. The method returns true value, so can be used in following constructs
         …
     };
 
+Messages are reported at C<error> level, if level is not explicitly specified. (It seems level is
+not used by C<Dist::Zilla> now.) Custom arguments are respected, too:
+
+    $self->log_error( { prefix => 'subsystem: ', level => 'info' }, $message );
+
+(See L<Log::Dispatchouli/log> for more details. Err… Unfortunately, there are very little details
+there.)
+
 =head2 abort_if_error
 
 If there was any errors (i. e. C<error_count> is greater than zero), the method aborts execution by
-calling C<log_fatal>.
+calling C<log_fatal> with all the arguments. If the method is called with no arguments, it pass
+to C<logh_fatal> string "Aborting...".
 
 =head1 AUTHOR
 
